# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

import json
import yaml

class Formatter():

    def format_matrix(self, matrix):
        return None

    def format_licenses(self, license_list):
        return None

    def format_verification(self, verification):
        return None

    def format_exception(self, exception):
        return None

    def format_compatiblity(self, compatibility):
        return None

    def _exception_to_dict(self, exception):
        return {
            'message': exception.error_message(),
            'code': exception.error_code(),
        }

    @staticmethod
    def formatter(_format):
        if _format.lower() == "json":
            return JsonFormatter()
        elif _format.lower() == "yaml" or _format.lower() == "yml":
            return YamlFormatter()
        elif _format.lower() == "csv":
            return CsvFormatter()
        elif _format.lower() == "text" or _format.lower() == "txt":
            return TxtFormatter()

class JsonFormatter(Formatter):

    def format_matrix(self, matrix):
        return json.dumps(matrix, indent=4)

    def format_licenses(self, license_list):
        return json.dumps(list(license_list), indent=4)

    def format_verification(self, verification):
        return json.dumps(verification)

    def format_exception(self, exception):
        return json.dumps(self._exception_to_dict(exception))

    def format_compatiblity(self, compatibility):
        return json.dumps(compatibility)

class CsvFormatter(Formatter):

    def __format_row_item(self, item):
        return f'"{item}"'

    def format_matrix(self, matrix):
        rows = []
        keys = list(matrix.keys())
        keys.remove('timeformat')
        keys.remove('timestamp')
        rows.append(f'"Compatiblity*", {", ".join([self.__format_row_item(x) for x in keys])}')

        for key in keys:
            row = []
            row.append(f'{key}')
            for inner_key in keys:
                row.append(f'{self.__format_row_item(matrix[key][inner_key])}')
            rows.append(', '.join(row))
        return '\n'.join(rows)

    def format_licenses(self, license_list):
        return {",".join(license_list)}

    def format_verification(self, verification):
        return f'{verification}'

    def format_exception(self, exception):
        return exception.error_message()

    def format_compatiblity(self, compatibility):
        return compatibility

class TxtFormatter(Formatter):

    def format_matrix(self, matrix):
        return CsvFormatter().format_matrix(matrix)

    def format_licenses(self, license_list):
        return '\n'.join(license_list)

    def format_verification(self, verification):
        return f'{verification}'

    def format_exception(self, exception):
        return exception.error_message()

    def format_compatiblity(self, compatibility):
        return compatibility

class YamlFormatter(Formatter):

    def format_matrix(self, matrix):
        return yaml.safe_dump(matrix)

    def format_licenses(self, license_list):
        return yaml.safe_dump(list(license_list))

    def format_verification(self, verification):
        return yaml.safe_dump(verification)

    def format_exception(self, exception):
        return yaml.safe_dump(self._exception_to_dict(exception))

    def format_compatiblity(self, compatibility):
        return yaml.safe_dump(compatibility)



#!/usr/bin/env python3

# SPDX-FileCopyrightText: 2023 Henrik Sandklef
#
# SPDX-License-Identifier: GPL-3.0-or-later

from argparse import RawTextHelpFormatter
import argparse
import logging
import sys
import traceback

import elmat.config
from elmat import Elmat
from elmat import ElmatException
from elmat import ElmatReturnCodes
from elmat.format import Formatter

def get_parser():

    parser = argparse.ArgumentParser(
        description="",
        epilog="",
        formatter_class=RawTextHelpFormatter,
    )
    parser.set_defaults(func=None)

    parser.add_argument('-of', '--output-format',
                        type=str,
                        help='Format for outoput',
                        default="JSON")

    parser.add_argument('--exclude-osadl', '-eo', action='store_true', dest='exclude_osadl', help='Exclude the licenses as found in the OSADL license matrix.', default=False)

    parser.add_argument('--exclude-elmat', '-ee', action='store_true', dest='exclude_elmat', help='Exclude the licenses as found in the Elmat license file.', default=False)

    parser.add_argument('--license-file', '-lf', dest='license_files', type=str, nargs='+', help='license files to merge')

    parser.add_argument('-V', '--version',
                        action='version',
                        version=elmat.config.SW_VERSION)

    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        help='output verbose information',
                        default=False)

    parser.add_argument('-d', '--debug',
                        action='store_true',
                        help='output debug information to stderr',
                        default=False)

    subparsers = parser.add_subparsers(help='Sub commands')

    # list
    parser_l = subparsers.add_parser(
        'list', help='List licenses')
    parser_l.set_defaults(which='list', func=list_licenses)

    # merge
    parser_m = subparsers.add_parser(
        'merge', help='Merge license and output result')
    parser_m.set_defaults(which='merge', func=merge_licenses)

    # verify
    parser_v = subparsers.add_parser(
        'verify', help='Verify if inbound license and an outbound license are compatible (true/false)')
    parser_v.set_defaults(which='merge', func=verify)
    parser_v.add_argument('--inbound-license', '-il', type=str, dest='inbound_license', help='', default=False)
    parser_v.add_argument('--outbound-license', '-ol', type=str, dest='outbound_license', help='', default=False)

    # compatibility
    parser_c = subparsers.add_parser(
        'compatibility', help='Get the compatiblity status between an inbound license and an outbound license')
    parser_c.set_defaults(which='compatiblity', func=compatibility)
    parser_c.add_argument('--inbound-license', '-il', type=str, dest='inbound_license', help='', default=False)
    parser_c.add_argument('--outbound-license', '-ol', type=str, dest='outbound_license', help='', default=False)

    return parser

def __get_elmat(args):
    include_osadl = not args.exclude_osadl
    include_elmat = not args.exclude_elmat
    license_files = args.license_files

    elmat = Elmat(license_files, include_osadl, include_elmat)
    return elmat

def compatibility(args, formatter):
    elmat = __get_elmat(args)

    inbound = args.inbound_license
    outbound = args.outbound_license
    if outbound not in elmat.supported_licenses():
        raise ElmatException(ElmatReturnCodes.ELMAT_LICENSE_UNKNOWN, f'Outbound license "{outbound}" not supported.')
    if inbound not in elmat.supported_licenses():
        raise ElmatException(ElmatReturnCodes.ELMAT_LICENSE_UNKNOWN, f'Inbound license "{inbound}" not supported.')

    ret = elmat.get_compatibility(outbound, inbound)
    formatted = formatter.format_compatiblity(ret)
    return formatted

def verify(args, formatter):
    elmat = __get_elmat(args)

    inbound = args.inbound_license
    outbound = args.outbound_license
    if outbound not in elmat.supported_licenses():
        raise ElmatException(ElmatReturnCodes.ELMAT_LICENSE_UNKNOWN, f'Outbound license "{outbound}" not supported.')
    if inbound not in elmat.supported_licenses():
        raise ElmatException(ElmatReturnCodes.ELMAT_LICENSE_UNKNOWN, f'Inbound license "{inbound}" not supported.')

    ret = elmat.is_compatible(outbound, inbound)
    formatted = formatter.format_verification(ret)
    return formatted


def list_licenses(args, formatter):
    elmat = __get_elmat(args)
    matrix = elmat.supported_licenses()
    formatted = formatter.format_licenses(matrix)
    return formatted

def merge_licenses(args, formatter):
    elmat = __get_elmat(args)
    matrix = elmat.matrix()
    formatted = formatter.format_matrix(matrix)
    return formatted

def main():

    parser = get_parser()
    args = parser.parse_args()
    formatter = Formatter.formatter(args.output_format)

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if args.func:
        try:
            ret = args.func(args, formatter)
            print(ret)
        except ElmatException as e:
            print(formatter.format_exception(e))
            sys.exit(e.error_code())
        except Exception as e:
            logging.debug(f'exception caught: {e}')
            if args.verbose:
                print(traceback.format_exc())
    else:
        parser.print_help(sys.stderr)


if __name__ == '__main__':
    main()
